@@ -10,6 +10,7 @@
 #include "desktopportal_debug.h"
 
 #include "dynamiclauncher.h"
+#include "globalshortcuts.h"
 
 DesktopPortal::DesktopPortal(QObject *parent)
     : QObject(parent)
@@ -26,7 +27,9 @@ DesktopPortal::DesktopPortal(QObject *parent)
 {
     const QByteArray xdgCurrentDesktop = qgetenv("XDG_CURRENT_DESKTOP");
     if (xdgCurrentDesktop.compare("KDE", Qt::CaseInsensitive) == 0) {
-        m_background = new BackgroundPortal(this);
+        new GlobalShortcutsPortal(this);
+
+        m_background = new BackgroundPortal(this, this);
         m_screenCast = new ScreenCastPortal(this);
         m_remoteDesktop = new RemoteDesktopPortal(this);
         m_screenshot = new ScreenshotPortal(this);
