@@ -7,7 +7,6 @@
 
 #include "notification.h"
 
-#include <QDBusArgument>
 #include <QDBusConnection>
 #include <QDBusMessage>
 #include <QDBusMetaType>
@@ -82,14 +81,11 @@ void NotificationPortal::AddNotification(const QString &app_id, const QString &i
     QList<Action> actionValues = {
         {notification.value(QStringLiteral("default-action")).toString(), notification.value(QStringLiteral("default-action-target"))}};
     if (notification.contains(QStringLiteral("buttons"))) {
-        QList<QVariantMap> buttons;
-        QDBusArgument dbusArgument = notification.value(QStringLiteral("buttons")).value<QDBusArgument>();
-        while (!dbusArgument.atEnd()) {
-            dbusArgument >> buttons;
-        }
+        const QDBusArgument dbusArgument = notification.value(QStringLiteral("buttons")).value<QDBusArgument>();
+        const auto buttons = qdbus_cast<QList<QVariantMap>>(dbusArgument);
 
         QStringList actions;
-
+        actions.reserve(buttons.count());
         for (const QVariantMap &button : qAsConst(buttons)) {
             actions << button.value(QStringLiteral("label")).toString();
 
@@ -153,9 +149,9 @@ void NotificationPortal::notificationActivated(uint action)
         params += ourAction.second;
     }
 
-    OrgFreedesktopApplicationInterface iface("org.freedesktop.Application", appPathFromId(appId), QDBusConnection::sessionBus());
-    if (id.startsWith("app.") && iface.isValid()) {
-        iface.ActivateAction(id.mid(4), params, platformData);
+    OrgFreedesktopApplicationInterface iface(appId, appPathFromId(appId), QDBusConnection::sessionBus());
+    if (ourAction.first.startsWith("app.") && iface.isValid()) {
+        iface.ActivateAction(ourAction.first.mid(4), params, platformData);
     } else {
         if (iface.isValid()) {
             iface.Activate(platformData);
