@@ -7,8 +7,8 @@
  */
 
 #include "screenshotdialog.h"
+#include "screenshotdialog_debug.h"
 
-#include <QLoggingCategory>
 #include <QPushButton>
 
 #include <KLocalizedString>
@@ -27,8 +27,6 @@
 #include <poll.h>
 #include <unistd.h>
 
-Q_LOGGING_CATEGORY(XdgDesktopPortalKdeScreenshotDialog, "xdp-kde-screenshot-dialog")
-
 static int readData(int fd, QByteArray &data)
 {
     char buffer[4096];
