@@ -8,6 +8,7 @@
  */
 
 #include "waylandintegration.h"
+#include "qwayland-wayland.h"
 #include "screencast.h"
 #include "screencasting.h"
 #include "waylandintegration_debug.h"
@@ -23,6 +24,7 @@
 #include <QScreen>
 #include <QThread>
 #include <QTimer>
+#include <QWaylandClientExtensionTemplate>
 
 #include <KLocalizedString>
 
@@ -37,10 +39,14 @@
 
 // system
 #include <fcntl.h>
+#include <linux/input-event-codes.h>
+#include <sys/mman.h>
 #include <unistd.h>
+#include <xkbcommon/xkbcommon.h>
 
-#include <KStatusNotifierItem>
 #include <KWayland/Client/fakeinput.h>
+#include <qpa/qplatformnativeinterface.h>
+#include <waylandintegration_debug.h>
 
 Q_GLOBAL_STATIC(WaylandIntegration::WaylandIntegrationPrivate, globalWaylandIntegration)
 
@@ -89,11 +95,6 @@ void WaylandIntegration::init()
     globalWaylandIntegration->initWayland();
 }
 
-void WaylandIntegration::authenticate()
-{
-    globalWaylandIntegration->authenticate();
-}
-
 bool WaylandIntegration::isStreamingEnabled()
 {
     return globalWaylandIntegration->isStreamingEnabled();
@@ -159,11 +160,21 @@ void WaylandIntegration::requestPointerAxisDiscrete(Qt::Orientation axis, qreal
     globalWaylandIntegration->requestPointerAxisDiscrete(axis, delta);
 }
 
+void WaylandIntegration::requestPointerAxis(qreal x, qreal y)
+{
+    globalWaylandIntegration->requestPointerAxis(x, y);
+}
+
 void WaylandIntegration::requestKeyboardKeycode(int keycode, bool state)
 {
     globalWaylandIntegration->requestKeyboardKeycode(keycode, state);
 }
 
+void WaylandIntegration::requestKeyboardKeysym(int keysym, bool state)
+{
+    globalWaylandIntegration->requestKeyboardKeysym(keysym, state);
+}
+
 void WaylandIntegration::requestTouchDown(quint32 touchPoint, const QPointF &pos)
 {
     globalWaylandIntegration->requestTouchDown(touchPoint, pos);
@@ -253,6 +264,7 @@ bool WaylandIntegration::WaylandIntegrationPrivate::isStreamingAvailable() const
 void WaylandIntegration::WaylandIntegrationPrivate::acquireStreamingInput(bool acquire)
 {
     if (acquire) {
+        authenticate();
         ++m_streamInput;
     } else {
         Q_ASSERT(m_streamInput > 0);
@@ -264,12 +276,7 @@ WaylandIntegration::Stream WaylandIntegration::WaylandIntegrationPrivate::startS
                                                                                                Screencasting::CursorMode cursorMode)
 {
     auto uuid = win[KWayland::Client::PlasmaWindowModel::Uuid].toString();
-    QString iconName = win[Qt::DecorationRole].value<QIcon>().name();
-    iconName = iconName.isEmpty() ? QStringLiteral("applications-all") : iconName;
-    return startStreaming(m_screencasting->createWindowStream(uuid, cursorMode),
-                          iconName,
-                          i18n("Recording window \"%1\"...", win[Qt::DisplayRole].toString()),
-                          {{QLatin1String("source_type"), static_cast<uint>(ScreenCastPortal::Window)}});
+    return startStreaming(m_screencasting->createWindowStream(uuid, cursorMode), {{QLatin1String("source_type"), static_cast<uint>(ScreenCastPortal::Window)}});
 }
 
 WaylandIntegration::Stream WaylandIntegration::WaylandIntegrationPrivate::startStreamingOutput(quint32 outputName, Screencasting::CursorMode mode)
@@ -277,8 +284,6 @@ WaylandIntegration::Stream WaylandIntegration::WaylandIntegrationPrivate::startS
     auto output = m_outputMap.value(outputName).output();
     m_streamedScreenPosition = output->globalPosition();
     return startStreaming(m_screencasting->createOutputStream(output.data(), mode),
-                          QStringLiteral("monitor"),
-                          i18n("Recording screen \"%1\"...", output->model()),
                           {
                               {QLatin1String("size"), output->pixelSize()},
                               {QLatin1String("source_type"), static_cast<uint>(ScreenCastPortal::Monitor)},
@@ -294,8 +299,6 @@ WaylandIntegration::Stream WaylandIntegration::WaylandIntegrationPrivate::startS
     }
     m_streamedScreenPosition = workspace.topLeft();
     return startStreaming(m_screencasting->createRegionStream(workspace, 1, mode),
-                          QStringLiteral("monitor"),
-                          i18n("Recording workspace..."),
                           {
                               {QLatin1String("size"), workspace.size()},
                               {QLatin1String("source_type"), static_cast<uint>(ScreenCastPortal::Monitor)},
@@ -306,18 +309,13 @@ WaylandIntegration::Stream
 WaylandIntegration::WaylandIntegrationPrivate::startStreamingVirtualOutput(const QString &name, const QSize &size, Screencasting::CursorMode mode)
 {
     return startStreaming(m_screencasting->createVirtualOutputStream(name, size, 1, mode),
-                          QStringLiteral("monitor"),
-                          i18n("Recording virtual output '%1'...", name),
                           {
                               {QLatin1String("size"), size},
                               {QLatin1String("source_type"), static_cast<uint>(ScreenCastPortal::Virtual)},
                           });
 }
 
-WaylandIntegration::Stream WaylandIntegration::WaylandIntegrationPrivate::startStreaming(ScreencastingStream *stream,
-                                                                                         const QString &iconName,
-                                                                                         const QString &description,
-                                                                                         const QVariantMap &streamOptions)
+WaylandIntegration::Stream WaylandIntegration::WaylandIntegrationPrivate::startStreaming(ScreencastingStream *stream, const QVariantMap &streamOptions)
 {
     QEventLoop loop;
     Stream ret;
@@ -345,25 +343,6 @@ WaylandIntegration::Stream WaylandIntegration::WaylandIntegrationPrivate::startS
         Q_ASSERT(ret.isValid());
         qDebug() << "start streaming" << ret << m_streamedScreenPosition;
 
-        auto item = new KStatusNotifierItem(stream);
-        item->setStandardActionsEnabled(false);
-        item->setTitle(description);
-        item->setIconByName(iconName);
-        item->setOverlayIconByName(QStringLiteral("media-record"));
-        item->setStatus(KStatusNotifierItem::Active);
-        connect(item, &KStatusNotifierItem::activateRequested, stream, [item] {
-            item->contextMenu()->show();
-        });
-        auto menu = new QMenu;
-        auto stopAction =
-            new QAction(QIcon::fromTheme(QStringLiteral("process-stop")), i18nc("@action:inmenu stops screen/window recording", "Stop Recording"));
-        connect(stopAction, &QAction::triggered, this, [this, nodeid, stream] {
-            stopStreaming(nodeid);
-            stream->deleteLater();
-        });
-        menu->addAction(stopAction);
-        item->setContextMenu(menu);
-
         loop.quit();
     });
     QTimer::singleShot(3000, &loop, &QEventLoop::quit);
@@ -421,6 +400,17 @@ void WaylandIntegration::WaylandIntegrationPrivate::requestPointerAxisDiscrete(Q
         m_fakeInput->requestPointerAxis(axis, delta);
     }
 }
+void WaylandIntegration::WaylandIntegrationPrivate::requestPointerAxis(qreal x, qreal y)
+{
+    if (m_streamInput && m_fakeInput) {
+        if (x != 0) {
+            m_fakeInput->requestPointerAxis(Qt::Horizontal, x);
+        }
+        if (y != 0) {
+            m_fakeInput->requestPointerAxis(Qt::Vertical, -y);
+        }
+    }
+}
 
 void WaylandIntegration::WaylandIntegrationPrivate::requestKeyboardKeycode(int keycode, bool state)
 {
@@ -433,6 +423,152 @@ void WaylandIntegration::WaylandIntegrationPrivate::requestKeyboardKeycode(int k
     }
 }
 
+namespace
+{
+struct XKBStateDeleter {
+    void operator()(struct xkb_state *state) const
+    {
+        return xkb_state_unref(state);
+    }
+};
+struct XKBKeymapDeleter {
+    void operator()(struct xkb_keymap *keymap) const
+    {
+        return xkb_keymap_unref(keymap);
+    }
+};
+struct XKBContextDeleter {
+    void operator()(struct xkb_context *context) const
+    {
+        return xkb_context_unref(context);
+    }
+};
+using ScopedXKBState = std::unique_ptr<struct xkb_state, XKBStateDeleter>;
+using ScopedXKBKeymap = std::unique_ptr<struct xkb_keymap, XKBKeymapDeleter>;
+using ScopedXKBContext = std::unique_ptr<struct xkb_context, XKBContextDeleter>;
+}
+class Xkb : public QtWayland::wl_keyboard
+{
+public:
+    struct Code {
+        const uint32_t level;
+        const uint32_t code;
+    };
+    std::optional<Code> keycodeFromKeysym(xkb_keysym_t keysym)
+    {
+        /* The offset between KEY_* numbering, and keycodes in the XKB evdev
+         * dataset. */
+        static const uint EVDEV_OFFSET = 8;
+
+        auto layout = xkb_state_serialize_layout(m_state.get(), XKB_STATE_LAYOUT_EFFECTIVE);
+        const xkb_keycode_t max = xkb_keymap_max_keycode(m_keymap.get());
+        for (xkb_keycode_t keycode = xkb_keymap_min_keycode(m_keymap.get()); keycode < max; keycode++) {
+            uint levelCount = xkb_keymap_num_levels_for_key(m_keymap.get(), keycode, layout);
+            for (uint currentLevel = 0; currentLevel < levelCount; currentLevel++) {
+                const xkb_keysym_t *syms;
+                uint num_syms = xkb_keymap_key_get_syms_by_level(m_keymap.get(), keycode, layout, currentLevel, &syms);
+                for (uint sym = 0; sym < num_syms; sym++) {
+                    if (syms[sym] == keysym) {
+                        return Code{currentLevel, keycode - EVDEV_OFFSET};
+                    }
+                }
+            }
+        }
+        return {};
+    }
+
+    static Xkb *self()
+    {
+        static Xkb self;
+        return &self;
+    }
+
+private:
+    Xkb()
+    {
+        m_ctx.reset(xkb_context_new(XKB_CONTEXT_NO_FLAGS));
+        if (!m_ctx) {
+            qCWarning(XdgDesktopPortalKdeWaylandIntegration) << "Failed to create xkb context";
+            return;
+        }
+        m_keymap.reset(xkb_keymap_new_from_names(m_ctx.get(), nullptr, XKB_KEYMAP_COMPILE_NO_FLAGS));
+        if (!m_keymap) {
+            qCWarning(XdgDesktopPortalKdeWaylandIntegration) << "Failed to create the keymap";
+            return;
+        }
+        m_state.reset(xkb_state_new(m_keymap.get()));
+        if (!m_state) {
+            qCWarning(XdgDesktopPortalKdeWaylandIntegration) << "Failed to create the xkb state";
+            return;
+        }
+
+        QPlatformNativeInterface *nativeInterface = qGuiApp->platformNativeInterface();
+        auto seat = static_cast<wl_seat *>(nativeInterface->nativeResourceForIntegration("wl_seat"));
+        init(wl_seat_get_keyboard(seat));
+    }
+
+    void keyboard_keymap(uint32_t format, int32_t fd, uint32_t size) override
+    {
+        if (format != WL_KEYBOARD_KEYMAP_FORMAT_XKB_V1) {
+            qCWarning(XdgDesktopPortalKdeWaylandIntegration) << "unknown keymap format:" << format;
+            close(fd);
+            return;
+        }
+
+        char *map_str = static_cast<char *>(mmap(nullptr, size, PROT_READ, MAP_SHARED, fd, 0));
+        if (map_str == MAP_FAILED) {
+            close(fd);
+            return;
+        }
+
+        m_keymap.reset(xkb_keymap_new_from_string(m_ctx.get(), map_str, XKB_KEYMAP_FORMAT_TEXT_V1, XKB_KEYMAP_COMPILE_NO_FLAGS));
+        munmap(map_str, size);
+        close(fd);
+
+        if (m_keymap)
+            m_state.reset(xkb_state_new(m_keymap.get()));
+        else
+            m_state.reset(nullptr);
+    }
+
+    ScopedXKBContext m_ctx;
+    ScopedXKBKeymap m_keymap;
+    ScopedXKBState m_state;
+};
+
+void WaylandIntegration::WaylandIntegrationPrivate::requestKeyboardKeysym(int keysym, bool state)
+{
+    if (m_streamInput && m_fakeInput) {
+        auto keycode = Xkb::self()->keycodeFromKeysym(keysym);
+        if (!keycode) {
+            qCWarning(XdgDesktopPortalKdeWaylandIntegration) << "Failed to convert keysym into keycode" << keysym;
+            return;
+        }
+
+        auto sendKey = [this, state](int keycode) {
+            if (state) {
+                m_fakeInput->requestKeyboardKeyPress(keycode);
+            } else {
+                m_fakeInput->requestKeyboardKeyRelease(keycode);
+            }
+        };
+        switch (keycode->level) {
+        case 0:
+            break;
+        case 1:
+            sendKey(KEY_LEFTSHIFT);
+            break;
+        case 2:
+            sendKey(KEY_RIGHTALT);
+            break;
+        default:
+            qCWarning(XdgDesktopPortalKdeWaylandIntegration) << "Unsupported key level" << keycode->level;
+            break;
+        }
+        sendKey(keycode->code);
+    }
+}
+
 void WaylandIntegration::WaylandIntegrationPrivate::requestTouchDown(quint32 touchPoint, const QPointF &pos)
 {
     if (m_streamInput && m_fakeInput) {
